{% for decorator in decorators -%}
{{ decorator }}
{% endfor -%}
@dataclass
{%- if base_class %}
class {{ class_name }}({{ base_class }}):
{%- else %}
class {{ class_name }}:
{%- endif %}
{%- if not fields %}
    pass
{%- endif %}
{%- for field in fields -%}
    {%- if field.default %}
    {{ field.name }}: {{ field.type_hint }} = {{field.default}}
    {%- else %}
    {{ field.name }}: {{ field.type_hint }}
    {%- endif %}
{%- endfor -%}