{% for decorator in decorators -%}
{{ decorator }}
{% endfor -%}
class {{ class_name }}({{ base_class }}):
{%- if not fields %}
    pass
{%- endif %}
{%- if config %}
    {% include 'Config.jinja2' %}
{%- endif %}
{%- for field in fields -%}
    {%- if field.field %}
    {{ field.name }}: {{ field.type_hint }} = {{ field.field }}
    {%- elif field.required %}
    {{ field.name }}: {{ field.type_hint }}
    {%- else %}
    {{ field.name }}: {{ field.type_hint }} = {{ field.default }}
    {%- endif %}
{%- endfor -%}
